using System.Text.Json.Serialization;
using OSMDatastructure;
using OSMDatastructure.Graph;
using Pathfinding;

var builder = WebApplication.CreateBuilder(args);

// Add services to the container.

builder.Services.AddControllers();
// Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
builder.Services.AddEndpointsApiExplorer();
builder.Services.AddSwaggerGen();

var app = builder.Build();

app.MapGet("/getRouteBeta", (float latStart, float lonStart, float latEnd, float lonEnd, Tag.SpeedType vehicle, double stayOnSameRoadPriority, double useHigherLevelRoadsPriority, double useRoadsWithLessJunctionsPriority) =>
    {
        DateTime startCalc = DateTime.Now;
        List<PathNode> result = Pathfinder.AStar("D:/stuttgart-regbez-latest", new Coordinates(latStart, lonStart),
            new Coordinates(latEnd, lonEnd), vehicle, useHigherLevelRoadsPriority, stayOnSameRoadPriority,
            useRoadsWithLessJunctionsPriority);
        PathResult pathResult = new PathResult(DateTime.Now - startCalc, result);
        return RenderPath.Renderer.DrawFromPath(result);
    } 
);

app.MapGet("/getRoute", (float latStart, float lonStart, float latEnd, float lonEnd, Tag.SpeedType vehicle, double stayOnSameRoadPriority, double useHigherLevelRoadsPriority, double useRoadsWithLessJunctionsPriority) =>
    {
        DateTime startCalc = DateTime.Now;
        List<PathNode> result = Pathfinder.AStar("D:/stuttgart-regbez-latest", new Coordinates(latStart, lonStart),
            new Coordinates(latEnd, lonEnd), vehicle, useHigherLevelRoadsPriority, stayOnSameRoadPriority,
            useRoadsWithLessJunctionsPriority);
        PathResult pathResult = new PathResult(DateTime.Now - startCalc, result);
        return pathResult;
    } 
);

app.MapGet("/getShortestRoute", (float latStart, float lonStart, float latEnd, float lonEnd) =>
    {
        DateTime startCalc = DateTime.Now;
        List<PathNode> result = Pathfinder.AStar("D:/stuttgart-regbez-latest", new Coordinates(latStart, lonStart),
            new Coordinates(latEnd, lonEnd), Tag.SpeedType.any, 0, 0,
            0);
        PathResult pathResult = new PathResult(DateTime.Now - startCalc, result);
        return pathResult;
    }
);

app.MapGet("/getClosestNode", (float lat, float lon) =>
{
    RegionManager regionManager = new RegionManager("D:/stuttgart-regbez-latest");
    return regionManager.ClosestNodeToCoordinates(new Coordinates(lat, lon), Tag.SpeedType.any);
});

// Configure the HTTP request pipeline.
if (app.Environment.IsDevelopment())
{
    app.UseSwagger();
    app.UseSwaggerUI();
}

app.UseHttpsRedirection();

app.UseAuthorization();

app.MapControllers();

app.Run();

internal class PathResult
{
    [JsonInclude]public TimeSpan calcTime;
    [JsonInclude]public List<PathNode> pathNodes;

    public PathResult(TimeSpan calcTime, List<PathNode> pathNodes)
    {
        this.calcTime = calcTime;
        this.pathNodes = pathNodes;
    }
}